@@ -144,11 +144,11 @@ QueryBuilder.prototype = {
 
         if (global) {
             globalSparql +=
-                (Global.categoryManager.getActiveCategoryWhere() +
+                Global.categoryManager.getActiveCategoryWhere() +
                  this._buildFilterString() +
                  ' } ' +
                  'ORDER BY DESC (?mtime)' +
-                 'LIMIT %d OFFSET %d').format(Global.offsetController.getOffsetStep(),
+                 ('LIMIT %d OFFSET %d').format(Global.offsetController.getOffsetStep(),
                                               Global.offsetController.getOffset());
         } else {
             globalSparql += this._buildFilterString() + ' }';
